﻿namespace ProjectMonorail;

public class DrawningMonorail
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityMonorail? EntityMonorail { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningWidth = 140;
    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningHeight = 60;
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="magneticRail">Признак наличия магнитного рельса</param>
    /// <param name="secondCabin">Признак наличия второй кабины</param>
    public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool magneticRail, bool secondCabin)
    {
        EntityMonorail = new EntityMonorail();
        EntityMonorail.Init(speed, weight, bodyColor, additionalColor, magneticRail, secondCabin);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя  разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (width >= _drawningWidth && height >= _drawningHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            return true;
        }
        return false;
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах

        if (x + 140 > _pictureWidth.Value)
        {
            x -= (x + 140) - _pictureWidth.Value;
        }
        if (y + 60 > _pictureHeight.Value)
        {
            y -= (y + 60) - _pictureHeight.Value;
        }

        _startPosX = x;
        _startPosY = y;
    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityMonorail == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityMonorail.Step > 0)
                {
                    _startPosX -= (int)EntityMonorail.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityMonorail.Step > 0)
                {
                    _startPosY -= (int)EntityMonorail.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityMonorail.Step + _drawningWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityMonorail.Step;
                }

                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityMonorail.Step + _drawningHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityMonorail.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityMonorail == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additionalBrush = new
        SolidBrush(EntityMonorail.AdditionalColor);

        //магнитная рельса
        if (EntityMonorail.MagneticRail)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value + 48, 130, 5);
        }

        //вторая кабина

        if (EntityMonorail.SecondCabin)
        {
            Point[] second_cabin = {
                new Point(_startPosX.Value + 125, _startPosY.Value),
                new Point(_startPosX.Value + 132, _startPosY.Value + 20),
                new Point(_startPosX.Value + 132, _startPosY.Value + 40),
                new Point(_startPosX.Value + 125, _startPosY.Value + 40),
                new Point(_startPosX.Value + 125, _startPosY.Value + 20),
                new Point(_startPosX.Value + 132, _startPosY.Value + 20)
            };
            g.DrawPolygon(pen, second_cabin);
            g.FillPolygon(additionalBrush, second_cabin);

        }

        // Границы
        Brush brush = new SolidBrush(EntityMonorail.BodyColor);
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 20, 120, 20);
        Point[] p1 = {
            new Point(_startPosX.Value + 12, _startPosY.Value),
            new Point(_startPosX.Value + 125, _startPosY.Value),
            new Point(_startPosX.Value + 125, _startPosY.Value + 20),
            new Point(_startPosX.Value + 5, _startPosY.Value + 20)
        };
        g.DrawPolygon(pen, p1);

        // Окрас

        g.FillRectangle(brush, _startPosX.Value + 6, _startPosY.Value + 21, 119, 19);

        Point[] p2 = {
            new Point(_startPosX.Value + 12, _startPosY.Value + 1),
            new Point(_startPosX.Value + 125, _startPosY.Value + 1),
            new Point(_startPosX.Value + 125, _startPosY.Value + 20),
            new Point(_startPosX.Value + 5, _startPosY.Value + 20)
        };
        g.FillPolygon(brush, p2);

        //дверь
        Brush brWhite = new SolidBrush(Color.White);
        g.DrawRectangle(pen, _startPosX.Value + 40, _startPosY.Value + 8, 10, 25);
        g.FillRectangle(brWhite, _startPosX.Value + 41, _startPosY.Value + 9, 9, 24);

        //окна
        Pen pen1 = new(Color.Blue);
        g.DrawRectangle(pen1, _startPosX.Value + 13, _startPosY.Value + 4, 10, 11);
        g.DrawRectangle(pen1, _startPosX.Value + 27, _startPosY.Value + 4, 10, 11);
        g.DrawRectangle(pen1, _startPosX.Value + 110, _startPosY.Value + 4, 10, 11);

        // держатели

        Brush brBlack = new SolidBrush(Color.Black);

        //1

        Point[] point1 = {
            new Point(_startPosX.Value + 7, _startPosY.Value + 40),
            new Point(_startPosX.Value + 45, _startPosY.Value + 40),
            new Point(_startPosX.Value + 38, _startPosY.Value + 46),
            new Point(_startPosX.Value, _startPosY.Value + 46)
        };
        g.DrawPolygon(pen, point1);
        g.FillPolygon(brBlack, point1);

        //2
        
        Point[] point3 = {
            new Point(_startPosX.Value + 123, _startPosY.Value + 40),
            new Point(_startPosX.Value + 130, _startPosY.Value + 46),
            new Point(_startPosX.Value + 92, _startPosY.Value + 46),
            new Point(_startPosX.Value + 85, _startPosY.Value + 40)
        };
        g.DrawPolygon(pen, point3);
        g.FillPolygon(brBlack, point3);


        //колеса
        
        //1-ое колеса
        g.DrawEllipse(pen, _startPosX.Value + 13, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 13, _startPosY.Value + 40, 11, 11);

        //2-ое колесо
        g.DrawEllipse(pen, _startPosX.Value + 35, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 35, _startPosY.Value + 40, 11, 11);

        //3-ее колесо
        g.DrawEllipse(pen, _startPosX.Value + 85, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 85, _startPosY.Value + 40, 11, 11);

        //4-ое колесо
        g.DrawEllipse(pen, _startPosX.Value + 107, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 107, _startPosY.Value + 40, 11, 11);
    }
}
